-- ToDo:
-- Trackersorting (impossible, because SortWatchFrame is something magically done)

QuestProgressSound = LibStub( "AceAddon-3.0"):NewAddon( "QuestProgressSound", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "LibSink-2.0")
local addonName,NS = ...
local Quixote = LibStub("LibQuixote-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuestProgressSound")
local LSM = LibStub("LibSharedMedia-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local LibQTip = LibStub("LibQTip-1.0")

local string,table,pairs,ipairs,strsplit,strlen,tonumber,floor,type =
			string,table,pairs,ipairs,strsplit,strlen,tonumber,floor,type

local VERSION = tostring(GetAddOnMetadata(addonName,"Version"))
local MAJOR_VERSION = (string.match(VERSION,"^(%d+)%..+"))

local STARTCOLOR = "|cFFD82619"
local MIDDLECOLOUR = "|cFFFFFF00"
local ENDCOLOR = "|cFF00FF19"
local TEXTCOLOR = "|cFFBFBFFF"
local COLORGREEN = "|cFF00FF00"
local COLORRED = "|cFFFF0000"
local COLORWHITE = "|cFFFFFFFF"
local COLORYELLOW = "|cFFFFFF00"

local LDB_ANCHOR

local pName = UnitName("player")

local options = {
	name = "QuestProgressSound",
	handler = QuestProgressSound,
	type = "group",
	childGroups = "tab",
	get = "getValue",
	set = "setValue",
	args = {
		player = {
			name = L["Player Sounds"],
			type = "group",
			args = {
				soundObjectiveUpdate = {
					name = L["Object Sound"],
					desc = L["Sound played when Part of Quest completed"],
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = LSM:HashTable("sound"),
				},
				soundQuestComplete = {
					name = L["Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed"],
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = LSM:HashTable("sound"),
				},
				soundpartyprogress = {
					name = L["Party Progress Sound"],
					desc = L["Sound played when Part of Quest completed by someone in your party"],
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = LSM:HashTable("sound"),
				},
				soundpartycomplete = {
					name = L["Party Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed by someone in your party"],
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = LSM:HashTable("sound"),
				},
			},
		},
		textnotify = {
			name = "Visual Party Progress",
			type = "group",
			args = {
				enableOutput = {
					name = L["Enable Output"],
					desc = L["Shows progress of party members"],
					type = "toggle",
				},
				output = QuestProgressSound:GetSinkAce3OptionsDataTable(),
			},
		},
		ldb = {
			name = "Data Broker",
			type = "group",
			args = {
				zonefontpath = {
					name = L["Zone Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = LSM:HashTable("font"),
				},
				zonefontsize = {
					name = L["Zone Font Size"],
					type = "range",
					min = 8,
					max = 24,
					step = 1,
				},
				questfontpath = {
					name = L["Objectives Font"],
					type = "select",
					dialogControl = 'LSM30_Font',
					values = LSM:HashTable("font"),
				},
				questfontsize = {
					name = L["Objectives Font Size"],
					type = "range",
					min = 8,
					max = 24,
					step = 1,
				},
			},
		},
	},
}

local defaults = {
	profile = {
		enableOutput = false,
		sinkoptions = {},
		soundObjectiveUpdate = 1,
		soundQuestComplete = 1,
		soundpartyprogress = 1,
		soundpartycomplete = 1,
		showzone = {},
		zonefontpath = nil,
		zonefontsize = 14,
		questfontpath = nil,
		questfontsize = 12,
	}
}

function QuestProgressSound:getValue(info)
	return self.db.profile[info[#info]]
end

function QuestProgressSound:setValue(info,value)
	self.db.profile[info[#info]] = value
end

function QuestProgressSound:OnInitialize()
	local sounds = {}
	sounds["Peon Yes"]				= "Sound\\Creature\\Peon\\PeonYes3.ogg"
	sounds["Peon What"]				= "Sound\\Creature\\Peon\\PeonWhat4.ogg"
	sounds["Peon Ready"]			= "Sound\\Creature\\Peon\\PeonReady1.ogg"
	sounds["Peon Complete"]		= "Sound\\Creature\\Peon\\PeonBuildingComplete1.ogg"
	sounds["Peasant What"]		= "Sound\\Creature\\Peasant\\PeasantWhat3.ogg"
	sounds["Peasant Ready"] 	= "Sound\\Creature\\Peasant\\PeasantReady1.ogg"
	sounds["Peasant Done"]		= "Interface\\AddOns\\QuestProgressSound\\Sounds\\Peasant_work_done.ogg"
	sounds["Millhouse Ready"]	= "Sound\\Creature\\MillhouseManastorm\\TEMPEST_Millhouse_Ready01.ogg"
	sounds["Millhouse Slay"]	= "Sound\\Creature\\MillhouseManastorm\\TEMPEST_Millhouse_Slay01.ogg"
	sounds["Ding"]						= "Interface\\AddOns\\QuestProgressSound\\Sounds\\ding.ogg"
	sounds["Tada"] 						= "Interface\\AddOns\\QuestProgressSound\\Sounds\\tada.ogg"
	sounds["Info"] 						= "Interface\\AddOns\\QuestProgressSound\\Sounds\\info.ogg"
	sounds["Alert"]						= "Interface\\AddOns\\QuestProgressSound\\Sounds\\Alert.ogg"
	for name, path in pairs( sounds) do
		LSM:Register( "sound", name, path)
	end

	self.db = LibStub("AceDB-3.0"):New("QuestProgressSoundDB", defaults, "Default")

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	self:SetSinkStorage(self.db.profile.sinkoptions)

	options.args.textnotify.args.output.disabled = function() return not self.db.profile.enableOutput end

	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestProgressSound", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestProgressSound","QuestProgressSound")
	self:RegisterChatCommand("qps", "ChatCommand")

	if self.db.profile.zonefontpath == nil then
		self.db.profile.zonefontpath = LSM:GetDefault("font")
		self.db.profile.zonefontsize = defaults.profile.zonefontsize
	end
	if self.db.profile.questfontpath == nil then
		self.db.profile.questfontpath = LSM:GetDefault("font")
		self.db.profile.questfontsize = defaults.profile.questfontsize
	end
	self:RegisterComm( "QPS", "OnCommReceive")
end

local part = {}
function QuestProgressSound:OnCommReceive( prefix, msg, how, who)
	-- 	self:Print( msg)
	if who == pName then return end
	part = table.wipe(part)
	part = {strsplit( "~", msg)}
	if not part[2] then return end
	if part[1] ~= MAJOR_VERSION then return end
	if part[2] == "objective" then
		self:Party_Objective_Update( who, part[3], part[4], part[5], part[6] )
	elseif part[2] == "complete" then
		self:Party_Quest_Complete( who, part[3])
	end
end

local partyUnits,tblWrap = {},{}
local MAX_PARTY_MEMBERS = MAX_PARTY_MEMBERS or 4
do
	for i=1,MAX_PARTY_MEMBERS do
		partyUnits[i] = string.format("party%d",i)
	end
end
function QuestProgressSound:showTooltip(tt)

	local tooltip = LibQTip:Acquire("QuestProgressSoundTooltip",4,"LEFT","CENTER","LEFT","RIGHT")

	local levelColumn = 1
	local watchColumn = 2
	local titleColumn = 3
	local groupColumn = 4
	tooltip:SetAutoHideDelay(0.25, tt)
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(tt)
	tooltip:Hide()
	tooltip:Clear()
	
	local headerFont = self.headerFont
	local questFont = self.questFont
	tooltip:SetHeaderFont(headerFont)
	tooltip:SetFont(questFont)
	local line
	for _, zone in Quixote:IterateZones() do
		local header_added = false
		local showzone = true
		if QuestProgressSound.db.profile.showzone[zone] ~= nil then
			showzone = QuestProgressSound.db.profile.showzone[zone]
		else
			QuestProgressSound.db.profile.showzone[zone] = true
		end
		if showzone then
			for _, uid, qid, title, level, tag, objectives, complete, group, daily, zone in Quixote:IterateQuestsInZone( zone) do
				if not header_added then
					header_added = self:addZoneToTooltip( tooltip, showzone, zone, levelColumn, headerFont)
				end
				line = tooltip:AddLine()
				local color = GetQuestDifficultyColor(level)
				local statustext = ""
				if complete==1 then
					statustext = string.format("%s (%s)",COLORGREEN,L["done"])
				elseif complete == -1 then
					statustext = string.format("%s (%s)",COLORRED,L["failed"])
				end
				tooltip:SetCell( line, levelColumn, string.format("%s[%s%s]",self:RGBtoText(color.r, color.g, color.b, 1),level,Quixote:GetShortTagForQuest(uid)))
				if self:isQuestWatched(qid) then
					tooltip:SetCell( line, watchColumn, "|TInterface\\Buttons\\UI-CheckBox-Check:0|t")
				end
				tooltip:SetCell( line, titleColumn, self:RGBtoText( color.r, color.g, color.b, 1) .. title .. statustext)
				tooltip:SetLineScript(line, "OnMouseUp", QuestProgressSound.Entry_OnMouseUp, qid)
				if objectives>0 then
					for objective, got, need, typ in Quixote:IterateObjectivesForQuest(uid) do
						if not got then got = 0 end
						if not need then need = 1 end
						local status = string.format("%s/%s",got,need)
						local colorstring
						if typ == 'reputation' then
							colorstring = self:GetColourString( Quixote:GetReactionLevel(got), Quixote:GetReactionLevel(need))
						else
							colorstring = self:GetColourString( got, need)
						end

						local showobjective = false
						if got ~= need then
							showobjective = true
						end
						local partyprogress = ""
						for i=1, GetNumGroupMembers() do
                            --self:Print("Checking "..i)
							local unit = partyUnits[i]
                            --self:Print("Unit is "..unit);
							local partygot = Quixote:GetPartyQuestObjective( i, uid, objective)
                            --if partygot then self:Print("Got "..partygot) else self:Print("got nothing") end
							if partygot and type(partygot) == 'number' and partygot ~= need then
								partyprogress = string.format("%s%s%s:%s%s", partyprogress, self:getClassColor(unit), (UnitName(unit)):sub(1,3), self:GetColourString(partygot,need), (partygot == need and 'd' or partygot) )   
							end
                        end
                        --self:Print("partyprogress " .. partyprogress)
						if strlen( partyprogress) > 0 then
							showobjective = true
						end
						if showobjective then
							line = tooltip:AddLine()
							tooltip:SetCell( line, titleColumn, string.format("%s %s%s",objective,colorstring,status))
							tooltip:SetCell( line, groupColumn, partyprogress)
						end
					end
				elseif (not complete or complete == 0) then
					local _, objectives = Quixote:GetQuestText(uid)
					if objectives and strlen(objectives)>50 then
						tblWrap = table.wipe(tblWrap)
						tblWrap = { strsplit(" ", objectives) }
						local text = ""
						for key,value in ipairs(tblWrap) do
							if strlen(text) + strlen(value) + 1 >= 50 then
								line = tooltip:AddLine()
								tooltip:SetCell( line, titleColumn, string.format("%s%s",COLORWHITE,text))
								text = value
							else
								if strlen(text)>0 then
									text = string.format("%s %s",text,value)
								else
									text = value
								end
							end
						end
						line = tooltip:AddLine()
						tooltip:SetCell( line, titleColumn, string.format("%s%s",COLORWHITE,text))
					elseif objectives then
						line = tooltip:AddLine()
						tooltip:SetCell( line, titleColumn, string.format("%s%s",COLORWHITE,objectives))
					end
				end
			end
			tooltip:AddLine()
		else
			header_added = self:addZoneToTooltip( tooltip, showzone, zone, levelColumn, headerFont)
		end
	end
	tooltip:AddLine(" ")
	line = tooltip:AddLine()
	line = tooltip:AddLine()
	tooltip:SetCell( line, levelColumn, string.format("%s%s%s - %s",COLORYELLOW,L["LeftClick"],COLORWHITE,L["Add Quest to Watchlist and set it as next to navigate to"]), "LEFT", 3)
	line = tooltip:AddLine()
	tooltip:SetCell( line, levelColumn, string.format("%s%s%s - %s",COLORYELLOW,L["RightClick"],COLORWHITE,L["Remove Quest from Watchlist"]), "LEFT", 3)
	line = tooltip:AddLine()
	tooltip:SetCell( line, levelColumn, string.format("%s%s+%s%s - %s",COLORYELLOW,L["Shift"],L["LeftClick"],COLORWHITE,L["Link Quest in Chat"]), "LEFT", 3)
	line = tooltip:AddLine()
	tooltip:SetCell( line, levelColumn, string.format("%s%s+%s%s - %s",COLORYELLOW,L["Shift"],L["RightClick"],COLORWHITE,L["Abandon Quest"]), "LEFT", 3)
	line = tooltip:AddLine()
	tooltip:SetCell( line, levelColumn, string.format("%s%s+%s%s - %s",COLORYELLOW,L["CTRL"],L["LeftClick"],COLORWHITE,L["Share Quest"]), "LEFT", 3)
	tooltip:AddLine(" ")
	
	tooltip:UpdateScrolling()
	tooltip:Show()
end

function QuestProgressSound:OnEnable()
	-- Called when the addon is enabled
	self:Print(L["Quest Progress Sounds loaded"])
	Quixote.RegisterCallback(self, "Objective_Update")
	Quixote.RegisterCallback(self, "Quest_Complete")

	Quixote.RegisterCallback(self, "Quest_Abandoned")
	Quixote.RegisterCallback(self, "Quest_Gained")
	Quixote.RegisterCallback(self, "Quest_Failed")
	Quixote.RegisterCallback(self, "Quest_Lost")
	Quixote.RegisterCallback(self, "Quest_Zone_Gained")
	Quixote.RegisterCallback(self, "Zone_Quests_Changed")
	Quixote.RegisterCallback(self, "Ready")
	
	local headerFont = CreateFont("qpsheaderFont")
	local questFont = CreateFont("qpsquestFont")
	headerFont:SetFont( LSM:Fetch( "font", self.db.profile.zonefontpath), self.db.profile.zonefontsize)
	questFont:SetFont( LSM:Fetch( "font", self.db.profile.questfontpath), self.db.profile.questfontsize)
	local headercolor = {}
	headercolor.r, headercolor.g, headercolor.b, headercolor.a = self:TextToRGB( "|cfff0b000")
	headerFont:SetTextColor( headercolor.r, headercolor.g, headercolor.b, headercolor.a)
	questFont:SetTextColor( 1, 1, 1, 1)
	self.headerFont = headerFont
	self.questFont = questFont
	
	self.dataobj = ldb:NewDataObject( "QuestProgressSound", {
		type = "data source",
		icon = [[Interface\Icons\INV_Misc_Book_04]],
		text = "Questlog",
		OnClick = function( clickedframe, button)
			if button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory( QuestProgressSound.optionsFrame)
			elseif button == "LeftButton" then
				ShowUIPanel(QuestLogFrame)
			end
		end,
		OnEnter = function(tt)
			LDB_ANCHOR = tt
			QuestProgressSound:showTooltip(tt)
		end,
		OnLeave = function(tt)
		end,
		}
	)
	
end

function QuestProgressSound:OnDisable()
	Quixote.UnregisterAll( self)
end

function QuestProgressSound:Quest_Abandoned()
	self:LDBTextUpdate()
end

function QuestProgressSound:Quest_Gained()
	self:LDBTextUpdate()
end

function QuestProgressSound:Quest_Failed()
	self:LDBTextUpdate()
end

function QuestProgressSound:Quest_Lost()
	self:LDBTextUpdate()
end

function QuestProgressSound:Quest_Zone_Gained()
	self:LDBTextUpdate()
end

function QuestProgressSound:Zone_Quests_Changed()
	self:LDBTextUpdate()
end

function QuestProgressSound:Ready()
	self:LDBTextUpdate()
end

function QuestProgressSound:Objective_Update(eventname, questtitle,uid,desc,old,currentobjects,neededobjects,type)
	local _,_,_,_,_,_,complete = Quixote:GetQuestByUid(uid)
	if (complete == nil) and (currentobjects==neededobjects) then
		PlaySoundFile( LSM:Fetch( "sound", self.db.profile.soundObjectiveUpdate))
	end
	if GetNumGroupMembers()>0 and complete==nil then
		local msg = string.format("%s~objective~%s~%s~%s~%s",MAJOR_VERSION,questtitle,desc,currentobjects,neededobjects)
		self:SendCommMessage( "QPS", msg, "PARTY")
	end
end

function QuestProgressSound:Quest_Complete( eventname, questtitle)
	PlaySoundFile( LSM:Fetch( "sound", self.db.profile.soundQuestComplete))
	self:LDBTextUpdate()
	if GetNumGroupMembers()>0 then
		local msg = string.format("%s~complete~%s",MAJOR_VERSION,questtitle)
		self:SendCommMessage( "QPS", msg, "PARTY")
	end
end

function QuestProgressSound:Party_Objective_Update( sender, questtitle, objective, numGot, numNeeded)
	if (self.db.profile.enableOutput) then
		self:Pour(string.format("%s %s: %s %s/%s %s",L["Questprogress of "],sender,questtitle,numGot,numNeeded,objective))
	end
	PlaySoundFile( LSM:Fetch( "sound", self.db.profile.soundpartyprogress))
end

function QuestProgressSound:Party_Quest_Complete( sender, questtitle)
	if (self.db.profile.enableOutput) then
		self:Pour(string.format("%s %s %s",sender,L["completed"],questtitle))
	end
	PlaySoundFile( LSM:Fetch( "sound", self.db.profile.soundpartycomplete))
end

function QuestProgressSound:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function QuestProgressSound:TextToRGB(strColour)
	local i = 3;
	local iAlpha = tonumber(string.sub(strColour, i, i + 1), 16);
	local iRed = tonumber(string.sub(strColour, i + 2, i + 3), 16);
	local iGreen = tonumber(string.sub(strColour, i + 4, i + 5), 16);
	local iBlue = tonumber(string.sub(strColour, i + 6, i + 7), 16);
	iAlpha = iAlpha / 255;
	iRed = iRed / 255;
	iGreen = iGreen / 255;
	iBlue = iBlue / 255;
	return iRed, iGreen, iBlue, iAlpha;
end

function QuestProgressSound:RGBtoText(iRed, iGreen, iBlue, iAlpha)
	if(iRed > 1.0) then iRed = 1.0; end
	if(iGreen > 1.0) then iGreen = 1.0; end
	if(iBlue > 1.0) then iBlue = 1.0; end
	if(iRed < 0.0) then iRed = 0.0; end
	if(iGreen < 0.0) then iGreen = 0.0; end
	if(iBlue < 0.0) then iBlue = 0.0; end

	iAlpha = floor(iAlpha * 255);
	iRed = floor(iRed * 255);
	iGreen = floor(iGreen * 255);
	iBlue = floor(iBlue * 255);
	return format("|c%2x%2x%2x%2x", iAlpha, iRed, iGreen, iBlue);
end

local colour,colourInitial,colourMid,colourComplete = {a=1.0},{},{},{};
			colourInitial.r, colourInitial.g, colourInitial.b, colourInitial.a = QuestProgressSound:TextToRGB(STARTCOLOR);
			colourMid.r, colourMid.g, colourMid.b, colourMid.a = QuestProgressSound:TextToRGB(MIDDLECOLOUR);
			colourComplete.r, colourComplete.g, colourComplete.b, colourComplete.a = QuestProgressSound:TextToRGB(ENDCOLOR);
local colourDelta1 = {
			a = (colourMid.a - colourInitial.a),r = (colourMid.r - colourInitial.r),g = (colourMid.g - colourInitial.g),b = (colourMid.b - colourInitial.b)};
local colourDelta2 = {
			a = (colourComplete.a - colourMid.a),r = (colourComplete.r - colourMid.r),g = (colourComplete.g - colourMid.g),b = (colourComplete.b - colourMid.b)};
function QuestProgressSound:GetColourString(NumItems, NumNeeded)
	
	if (NumNeeded == 0) then
		colour.r = 0;
		colour.g = 0;
		colour.b = 0;
	elseif ((NumItems / NumNeeded) < 0.5) then
		colour.r = colourInitial.r + ((NumItems / (NumNeeded / 2)) * colourDelta1.r);
		colour.g = colourInitial.g + ((NumItems / (NumNeeded / 2)) * colourDelta1.g);
		colour.b = colourInitial.b + ((NumItems / (NumNeeded / 2)) * colourDelta1.b);
	else
		colour.r = colourMid.r + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.r);
		colour.g = colourMid.g + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.g);
		colour.b = colourMid.b + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.b);
	end

	if(colour.r > 1.0) then colour.r = 1.0; end
	if(colour.g > 1.0) then colour.g = 1.0; end
	if(colour.b > 1.0) then colour.b = 1.0; end
	if(colour.r < 0.0) then colour.r = 0.0; end
	if(colour.g < 0.0) then colour.g = 0.0; end
	if(colour.b < 0.0) then colour.b = 0.0; end
	
	return self:RGBtoText(colour.r, colour.g, colour.b, colour.a);
end

function QuestProgressSound:getClassColor( sender )
	local localizedclass, englishclass = UnitClass(sender)
	local c = RAID_CLASS_COLORS[englishclass]
	if (c) then
		c = format("|c00%02X%02X%02X", 255 * c.r, 255 * c.g, 255 * c.b)
	else
		c = "|c00808080"
	end
	return c
end

local MAX_QUESTS = MAX_QUESTS or 25
function QuestProgressSound:LDBTextUpdate()
	local numQuest, numComplete = Quixote:GetNumQuests()
	QuestProgressSound.dataobj.text = string.format( "Questlog (%d/%d/%s%d%s)", numQuest, MAX_QUESTS, COLORGREEN, numComplete, COLORWHITE)
end

function QuestProgressSound:Entry_OnMouseUp( qid, button)
	if IsControlKeyDown() then
		if button == "LeftButton" then -- CTRL+Left Share Quest
			local currentlySelected = GetQuestLogSelection()
			SelectQuestLogEntry(qid)
			if GetQuestLogPushable() and GetNumGroupMembers() > 0 then
				QuestLogPushQuest()
			end
			SelectQuestLogEntry(currentlySelected)
		end
	elseif IsShiftKeyDown() then
		if button == "RightButton" then -- Shift+Right Abandon Quest
			local selected = GetQuestLogSelection()
			SelectQuestLogEntry(qid)
			local index = GetQuestLogSelection()
			SetAbandonQuest()
			local items = GetAbandonQuestItems()
			if items then
				StaticPopup_Hide("ABANDON_QUEST")
				StaticPopup_Show("ABANDON_QUEST_WITH_ITEMS", GetAbandonQuestName(), items)
			else
				StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS")
				StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName())
			end
			if index ~= selected then
				SelectQuestLogEntry(selected)
			end
		elseif button == "LeftButton" then -- Shift+Left Link Quest in Chat
			local edit_box = ChatEdit_ChooseBoxForSend()
			ChatEdit_ActivateChat(edit_box)
			edit_box:Insert(GetQuestLink(qid))
		end
	else
		if button == "RightButton" then -- Right Remove Quest from Watchlist
			local selected = GetQuestLogSelection()
			SelectQuestLogEntry(qid)
			local index = GetQuestLogSelection()
			if index then
				RemoveQuestWatch(index)
				if index ~= selected then
					SelectQuestLogEntry(selected)
				end
			end
		elseif button == "LeftButton" then -- Left Add Quest to Watchlist and set it as next to navigate to
			SelectQuestLogEntry(qid)
			local selected = GetQuestLogSelection()
			if not IsQuestWatched( selected) then
				AddQuestWatch(selected)
			end
		end
	end
	QuestProgressSound:showTooltip( LDB_ANCHOR)
end

function QuestProgressSound:Zone_OnMouseUp( zone, button)
	if QuestProgressSound.db.profile.showzone[zone] then
		QuestProgressSound.db.profile.showzone[zone] = false
	else
		QuestProgressSound.db.profile.showzone[zone] = true
	end
	QuestProgressSound:showTooltip( LDB_ANCHOR)
end

function QuestProgressSound:isQuestWatched( qid)
	local selected = GetQuestLogSelection()
	SelectQuestLogEntry(qid)
	local index = GetQuestLogSelection()
	local retValue = IsQuestWatched( index)
	SelectQuestLogEntry( selected)
	return retValue
end

function QuestProgressSound:addZoneToTooltip( tooltip, showzone, zone, levelColumn, headerFont)
	local MINIMIZEBUTTON	= "|TInterface\\BUTTONS\\UI-PlusButton-Up:0|t "
	local MAXIMIZEBUTTON	= "|TInterface\\BUTTONS\\UI-MinusButton-Up:0|t "
	local collapse = MINIMIZEBUTTON
	if showzone then
		collapse = MAXIMIZEBUTTON
	end
	local line = tooltip:AddHeader()
	tooltip:SetCell( line, levelColumn, string.format("%s%s",collapse,zone), headerFont, "LEFT", 3)
	tooltip:SetLineScript(line, "OnMouseUp", QuestProgressSound.Zone_OnMouseUp, zone)
	tooltip:AddLine()
	return true
end